# -*- coding: utf-8 -*-
"""
cms forms for acomea.app.switch_programmato app
"""
import datetime
from django import forms
from django.conf import settings
from django.forms.formsets import formset_factory, BaseFormSet
from django.forms.models import BaseInlineFormSet, inlineformset_factory
from django.utils.translation import ugettext as _

from jmb.core.utils.cmsutils import get_abs_url_from_slug

from acomea.app.soap import OpServizi
from acomea.app.models import Applicazione, Servizio, PosizioneFrom


def _get_choices(lista, empty=False):
    choices = []

    if empty:
        choices = [('', '---')]

    for num, label in enumerate(lista):
        choices.append((num, label))
    return choices


def _get_pianis_choices(lista, empty=False):
    choices = []

    if empty:
        choices = [('', '---')]

    for num, label in enumerate(lista):
        choices.append((label['ID'], label))
    return choices


def _get_gruppi_fondi_cardie(lista, empty=False):
    choices = []
    funds_data = {}
    if empty:
        choices = [('', '---')]
    for num, label in enumerate(lista):
        choices.append((label['CODFON'], label['DESCRFON']))
        funds_data[label['CODFON']] = {
            'DESCRFON': label.get('DESCRFON'),
            'CODFON': label.get('CODFON'),
            'CLASSE': label.get('CLASSE')
        }
    return choices, funds_data


MESI = (
    (0, 'Mese'),
    (1, 'Gennaio',), (2, 'Febbraio',), (3, 'Marzo',),
    (4, 'Aprile'), (5, 'Maggio',), (6, 'Giugno',),
    (7, 'Luglio',), (8, 'Agosto'), (9, 'Settembre',),
    (10, 'Ottobre',), (11, 'Novembre',), (12, 'Dicembre')
)


class ServizioFromForm(forms.Form):
    def _get_applicazione_choices(self):
        applicazioni = Applicazione.active_objects.all()
        choices = []
        modal_slugs = {}
        for app in applicazioni:
            choices.append((app.codice, app.get_codice_display(),))
            modal_slugs.update({ app.codice: get_abs_url_from_slug(app.modal_page_slug)})
        return (choices, modal_slugs)

    def __init__(self, *args, **kwargs):
        self.session = kwargs.pop('session', None)
        self.modal_slugs = None
        dati_servizio = self.session['app_dati_servizio']
        super(ServizioFromForm, self).__init__(*args, **kwargs)
        choices_positions_from_int = _get_choices(self.session['app_posizioni_intestatario'])
        choices_positions_from_coint = _get_choices(self.session['app_posizioni_cointestatario'])

        self.fields['tipo_operazione'] = forms.ChoiceField(
            widget=forms.RadioSelect(),
        )
        self.fields['tipo_operazione'].choices, self.modal_slugs = self._get_applicazione_choices()
        self.fields['posizione_from_int'] = forms.ChoiceField(
            choices=choices_positions_from_int,
            required=False,
            widget=forms.RadioSelect(),
        )
        self.fields['posizione_from_coint'] = forms.ChoiceField(
            choices=choices_positions_from_coint,
            required=False,
            widget=forms.RadioSelect(),
        )

    def clean(self):
        data = self.cleaned_data

        if not data['posizione_from_int'] and not data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare almeno un Rapporto/Contratto"))

        if data['posizione_from_int'] and data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare solo un Rapporto/Contratto tra intestatario e cointestatario"))

        return data


class ServizioMultiFromForm(forms.Form):
    PosizioneFromFormSet = inlineformset_factory(Servizio, PosizioneFrom)
    def _get_applicazione_choices(self):
        applicazioni = Applicazione.active_objects.all()
        choices = []
        modal_slugs = {}
        for app in applicazioni:
            choices.append((app.codice, app.description,))
            modal_slugs.update({ app.codice: get_abs_url_from_slug(app.modal_page_slug)})
        return (choices, modal_slugs)

    def __init__(self, *args, **kwargs):
        self.session = kwargs.pop('session', None)
        self.modal_slugs = None
        dati_servizio = self.session['app_dati_servizio']
        super(ServizioMultiFromForm, self).__init__(*args, **kwargs)
        choices_positions_from_int = _get_choices(self.session['app_posizioni_intestatario'])
        choices_positions_from_coint = _get_choices(self.session['app_posizioni_cointestatario'])

        self.fields['tipo_operazione'] = forms.ChoiceField(
            widget=forms.RadioSelect(),
        )
        self.fields['tipo_operazione'].choices, self.modal_slugs = self._get_applicazione_choices()
        self.fields['posizione_from_int'] = forms.MultipleChoiceField(
            choices=choices_positions_from_int,
            required=False,
            widget=forms.CheckboxSelectMultiple(),
        )
        self.fields['posizione_from_coint'] = forms.MultipleChoiceField(
            choices=choices_positions_from_coint,
            required=False,
            widget=forms.CheckboxSelectMultiple(),
        )

    def controlli_posizione_from(self, tipo_operazione, dettaglio_from):
        if tipo_operazione == "CARDIE" or tipo_operazione == "RIBINV":
            op_servizi = OpServizi()
            result = op_servizi.op_servizi_controlli_posizione_from(tipo_operazione, dettaglio_from)

            if not result['OK']:
                raise forms.ValidationError(result['ErrorMessage'])

    def create_lista_posizioni_from(self):
        data = self.cleaned_data
        lista_posizioni_from = []
        if 'posizione_from_int' in data:
            for ind in data['posizione_from_int']:
                posizione = {}
                posizione_scelta = self.session['app_posizioni_intestatario'][int(ind)]
                posizione['codice_rapporto_from'] = posizione_scelta['RAPPORTO']
                posizione['codice_deposito_from'] = posizione_scelta['NUMCONTRATTO']
                posizione['codice_fondo_from'] = posizione_scelta['CODFONDO']
                posizione['descrizione_fondo_from'] = posizione_scelta['DESCRFONDO']
                posizione['codice_tipocontratto_from'] = posizione_scelta['CODTC']
                posizione['tipo_piano_from'] = posizione_scelta['TIPOPIANO']
                posizione['patrimonio_from'] = posizione_scelta['PATRIMONIO']
                posizione['quote_from'] = posizione_scelta['QUOTE']
                lista_posizioni_from.append(posizione)
        if 'posizione_from_coint' in data:
            for ind in data['posizione_from_coint']:
                posizione = {}
                posizione_scelta = self.session['app_posizioni_cointestatario'][int(ind)]
                posizione['codice_rapporto_from'] = posizione_scelta['RAPPORTO']
                posizione['codice_deposito_from'] = posizione_scelta['NUMCONTRATTO']
                posizione['codice_fondo_from'] = posizione_scelta['CODFONDO']
                posizione['descrizione_fondo_from'] = posizione_scelta['DESCRFONDO']
                posizione['codice_tipocontratto_from'] = posizione_scelta['CODTC']
                posizione['tipo_piano_from'] = posizione_scelta['TIPOPIANO']
                posizione['patrimonio_from'] = posizione_scelta['PATRIMONIO']
                posizione['quote_from'] = posizione_scelta['QUOTE']
                lista_posizioni_from.append(posizione)
        return lista_posizioni_from

    def create_stringa_posizioni_from(self):
        lista = []
        for posizione in self.create_lista_posizioni_from():
            lista.append("%s;%s;%s;%s;%s" % (
                posizione['codice_rapporto_from'], posizione['codice_deposito_from'],
                posizione['codice_tipocontratto_from'], posizione['codice_fondo_from'], posizione['patrimonio_from']
            ))
        return "|".join(lista)

    def clean(self):
        data = self.cleaned_data
        if not data['posizione_from_int'] and not data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare almeno un Rapporto/Contratto"))
        if data['posizione_from_int'] and data['posizione_from_coint']:
            raise forms.ValidationError(_("Selezionare solo un Rapporto/Contratto tra intestatario e cointestatario"))
        if 'tipo_operazione' in data:
            self.controlli_posizione_from(data['tipo_operazione'], self.create_stringa_posizioni_from())
        return data



class ServizioToForm(forms.Form):
    def _get_anni(self):
        curr_anno = datetime.date.today().year
        anni = [(0, 'Anno')]
        for anno in range(curr_anno, curr_anno+5):
            anni.append((anno, str(anno)))
        return anni

    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        super(ServizioToForm, self).__init__(*args, **kwargs)

        self.dati_servizio = session['app_dati_servizio']
        self.servizio = Servizio.objects.get(id=self.dati_servizio['id'])
        self.session = session
        self.funds_data_a = None
        self.funds_data_b = None

        #################
        # ATTIVA CEDOLA #
        #################
        if self.dati_servizio['tipo_operazione'] == "ATTCED":
            self.posizioni_to = session['app_posizioni_to']
            choices_positions_to = _get_choices(self.posizioni_to)
            self.fields['posizione_to'] = forms.ChoiceField(
                choices=choices_positions_to,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['iban'] = forms.CharField(
                required=False, label="Iban di accredito", max_length=27,
            )

        ##############
        # CARPE DIEM #
        ##############
        if self.dati_servizio['tipo_operazione'] == "CARDIE":
            opservizi = OpServizi()
            id_servizio = self.dati_servizio['id']
            codice_fondo = PosizioneFrom.objects.filter(servizio_id=id_servizio)[0].codice_fondo_from
            lista_fondi_gruppo_a = opservizi.op_servizi_get_elenco_fondi_cardie(
                codice_fondo, 'A'
            )
            lista_fondi_gruppo_b = opservizi.op_servizi_get_elenco_fondi_cardie(
                codice_fondo, 'B'
            )
            choices_fondi_gruppo_a, self.funds_data_a = _get_gruppi_fondi_cardie(
                lista_fondi_gruppo_a
            )
            choices_fondi_gruppo_b, self.funds_data_b = _get_gruppi_fondi_cardie(
                lista_fondi_gruppo_b
            )

            choices_tcpic = [
                (tcpic["OpServizi_TCPIC"]["NumContratto"], tcpic["OpServizi_TCPIC"]["NumContrattoDescr"])
                for tcpic in session['elenco_tcpic']
            ]

            self.fields['car_die_gruppo_a'] = forms.ChoiceField(
                choices=choices_fondi_gruppo_a,
                required=True,
                widget=forms.RadioSelect(),
            )
            self.fields['car_die_gruppo_b'] = forms.ChoiceField(
                choices=choices_fondi_gruppo_b,
                required=True,
                widget=forms.RadioSelect(),
            )
            self.fields['tcpic'] = forms.ChoiceField(
                choices=choices_tcpic,
                initial=choices_tcpic[0][0],
                required=True,
                widget=forms.Select()
            )
            self.fields['car_die_allocazione_a'] = forms.IntegerField(
                label="Allocazione di partenza Gruppo A in %"
            )
            self.fields['car_die_allocazione_b'] = forms.IntegerField(
                label="Allocazione di partenza Gruppo B in %"
            )
            self.fields['car_die_perc_var_a'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_CAR_DIE_PERC_VAR_A, initial=0,
                label="Percentuale di variazione della quota del Fondo/comparto gruppo 'A'"
            )
            self.fields['car_die_perc_riall_ab'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_CAR_DIE_PERC_RIALL_AB, initial=0,
                label="Percentuale di riallocazione successiva tra i gruppi 'A' e 'B'%"
            )

        #####################
        # RADDOPPIA RIBASSO #
        #####################
        elif self.dati_servizio['tipo_operazione'] == "RADRIB":
            self.posizioni_pianis_rid = session['app_posizioni_pianis_rid']
            self.posizioni_pianis_swipro = session['app_posizioni_pianis_swipro']
            choices_posizioni_pianis_rid = _get_pianis_choices(self.posizioni_pianis_rid)
            choices_posizioni_pianis_swipro = _get_pianis_choices(self.posizioni_pianis_swipro)

            self.fields['posizioni_pianis_rid'] = forms.ChoiceField(
                choices=choices_posizioni_pianis_rid,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['posizioni_pianis_swipro'] = forms.ChoiceField(
                choices=choices_posizioni_pianis_swipro,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['fine_servizio'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
                widget=forms.RadioSelect(),
                initial="R"
            )
            self.fields['fine_servizio_anno'] = forms.ChoiceField(choices=self._get_anni(), required=False)
            self.fields['fine_servizio_mese'] = forms.ChoiceField(choices=MESI, required=False)
            self.fields['rad_rib_riduz_quota_a'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_RAD_RIB_RIDUZ_A, initial=0, label="Ipotesi A"
            )
            self.fields['rad_rib_riduz_quota_b'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_RAD_RIB_RIDUZ_B, initial=0, label="Ipotesi B"
            )
            # self.fields['mifid'] = forms.BooleanField(
            #     required=False
            # )

        ######################
        # REINVESTI PROFITTI #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "REIPRO":
            self.posizioni_to_ex = session['app_posizioni_to_ex']
            self.posizioni_to_not = session['app_posizioni_to_not']
            choices_positions_to_ex = _get_choices(self.posizioni_to_ex)
            choices_positions_to_not = _get_choices(self.posizioni_to_not)
            self.fields['posizione_to_ex'] = forms.ChoiceField(
                choices=choices_positions_to_ex,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['posizione_to_not'] = forms.ChoiceField(
                choices=choices_positions_to_not,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['fine_servizio'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
                widget=forms.RadioSelect(),
                initial="R"
            )
            self.fields['fine_servizio_anno'] = forms.ChoiceField(choices=self._get_anni(), required=False)
            self.fields['fine_servizio_mese'] = forms.ChoiceField(choices=MESI, required=False)
            self.fields['rei_pro_perc_plus'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_REI_PRO_PERC_PLUS, initial=0, label="Seleziona il valore % di plusvalenza da reinvestire"
            )

        ###########################
        # RIBILANCIA INVESTIMENTO #
        ###########################
        elif self.dati_servizio['tipo_operazione'] == "RIBINV":
            self.fondi_totali_to = session['app_fondi_to_totali']
            for fondo in self.fondi_totali_to:
                ind = "fondo_{}_{}_{}".format(fondo['CODFONDO'], fondo['RAPPORTO'], fondo['NUMCONTRATTO'])
                self.fields[ind] = forms.IntegerField(
                    initial=fondo['PESO'],
                    label="fondo {}".format(fondo['DESCRFONDO'])
                )
            self.fields['rib_inv_perc_scost'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_RIB_INV_PERC_SCOST,
                label="Percentuale di Scostamento",
                initial=0
            )
            self.fields['fine_servizio'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
                widget=forms.RadioSelect(),
                initial="R"
            )
            self.fields['fine_servizio_anno'] = forms.ChoiceField(choices=self._get_anni(), required=False)
            self.fields['fine_servizio_mese'] = forms.ChoiceField(choices=MESI, required=False)

        #########################
        # RIMBORSO PROGRAMMATO  #
        #########################
        elif self.dati_servizio['tipo_operazione'] == "RIMPRO":
            self.fields['importo'] = forms.DecimalField(required=False, decimal_places=2, localize=True)
            self.fields['quote'] = forms.DecimalField(required=False, decimal_places=3, localize=True)
            self.fields['tipo_qta_movimento'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_TIPO_QTA_MOVIMENTO_CHOICES,
                widget=forms.RadioSelect(),
                initial="I"
            )
            self.fields['frequenza'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FREQUENZA_CHOICES,
                initial="M"
            )
            self.fields['data_decorrenza'] = forms.DateField(
                required=True, localize=True
            )
            self.fields['fine_servizio'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
                widget=forms.RadioSelect(),
                initial="R"
            )
            self.fields['fine_servizio_anno'] = forms.ChoiceField(choices=self._get_anni(), required=False)
            self.fields['fine_servizio_mese'] = forms.ChoiceField(choices=MESI, required=False)
            self.fields['iban'] = forms.CharField(label="Iban di accredito", max_length=27)

        ######################
        # SWITCH PROGRAMMATO #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "SWIPRO":
            self.posizioni_to_ex = session['app_posizioni_to_ex']
            self.posizioni_to_not = session['app_posizioni_to_not']

            choices_positions_to_ex = _get_choices(self.posizioni_to_ex)
            choices_positions_to_not = _get_choices(self.posizioni_to_not)
            self.fields['posizione_to_ex'] = forms.ChoiceField(
                choices=choices_positions_to_ex,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['posizione_to_not'] = forms.ChoiceField(
                choices=choices_positions_to_not,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields['tipo_piano_not'] = forms.ChoiceField(
                choices=(('PIC', 'PIC'), ('PAD', 'PAD'),),
                widget=forms.RadioSelect(),
                required=False,
                initial='PIC'
            )
            self.fields['tipo_qta_movimento'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_TIPO_QTA_MOVIMENTO_CHOICES,
                widget=forms.RadioSelect(),
                initial="I"
            )
            self.fields['importo'] = forms.DecimalField(
                required=False,
                decimal_places=2,
                localize=True
            )
            self.fields['numero_rate_versate'] = forms.DecimalField(
                required=False,
                decimal_places=2,
                localize=True
            )
            self.fields['quote'] = forms.DecimalField(
                label="Numero quote",
                required=False,
                decimal_places=3,
                localize=True
            )
            self.fields['frequenza'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FREQUENZA_CHOICES,
                initial="M"
            )
            self.fields['data_decorrenza'] = forms.DateField(
                required=True, localize=True
            )
            self.fields['fine_servizio'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
                widget=forms.RadioSelect(),
                initial="R"
            )
            self.fields['fine_servizio_anno'] = forms.ChoiceField(choices=self._get_anni(), required=False)
            self.fields['fine_servizio_mese'] = forms.ChoiceField(choices=MESI, required=False)

            self.fields['rad_rib_flag'] = forms.BooleanField(initial=False, required=False)
            self.fields['rad_rib_riduz_quota_a'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_RAD_RIB_RIDUZ_A, initial=0, label="Ipotesi A"
            )
            self.fields['rad_rib_riduz_quota_b'] = forms.ChoiceField(
                choices=settings.APP_SERVIZIO_RAD_RIB_RIDUZ_B, initial=0, label="Ipotesi B"
            )

    def can_operate(self):
        data = self.cleaned_data
        op_servizi = OpServizi()
        mifid = 'N'
        iban = dettaglio_from = dettaglio_to = frequenza = ''
        valore = rei_pro_perc_plus = car_die_allocazione_a = car_die_allocazione_b = 0
        car_die_perc_var_a = car_die_perc_riall_ab = rad_rib_nop_pianis = 0
        rib_inv_perc_scost = numero_rate_versate = 0
        rad_rib_riduz_quota_a = rad_rib_riduz_quota_b = 0
        tipo_qta_movimento = "IMPORTO"
        fine_servizio = "DATA"
        fondo_gruppo_a = fondo_gruppo_b = 0

        rad_rib_flag = 'N'
        if 'rad_rib_flag' in data and data['rad_rib_flag']:
            rad_rib_flag = 'S'

        #################
        # ATTIVA CEDOLA #
        #################
        if self.dati_servizio['tipo_operazione'] == "ATTCED":
            rapporto = deposito = 0
            fondo = tipo_contratto = ""
            valore = numero_rate_versate = 0
            frequenza = ""
            data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
            fine_servizio = "REVOCA"
            fine_servizio_anno = fine_servizio_mese = 0

            if data['posizione_to']:
                indice = int(data['posizione_to'])
                rapporto = self.posizioni_to[indice]['RAPPORTO']
                deposito = self.posizioni_to[indice]['NUMCONTRATTO']
                fondo = self.posizioni_to[indice]['CODFONDO']
                tipo_contratto = self.posizioni_to[indice]['CODTC']

            iban = data['iban']

        ##############
        # CARPE DIEM #
        ##############
        elif self.dati_servizio['tipo_operazione'] == "CARDIE":
            rapporto = deposito = 0
            fondo = tipo_contratto = ""
            valore = numero_rate_versate = 0
            frequenza = ""
            data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
            fine_servizio = "REVOCA"
            fine_servizio_anno = fine_servizio_mese = 0
            fondo_gruppo_a = data['car_die_gruppo_a']
            fondo_gruppo_b = data['car_die_gruppo_b']
            deposito = data['tcpic']
            car_die_allocazione_a = data['car_die_allocazione_a']
            car_die_allocazione_b = data['car_die_allocazione_b']
            car_die_perc_var_a = data['car_die_perc_var_a']
            car_die_perc_riall_ab = data['car_die_perc_riall_ab']
            dettaglio_from = self.servizio.dettaglio_from

        #####################
        # RADDOPPIA RIBASSO #
        #####################
        elif self.dati_servizio['tipo_operazione'] == "RADRIB":
            valore = numero_rate_versate = 0
            frequenza = ""
            data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
            if data['fine_servizio'] == "R":
                fine_servizio = "REVOCA"
            fine_servizio_anno = data['fine_servizio_anno']
            fine_servizio_mese = data['fine_servizio_mese']

            rapporto = self.dati_servizio['codice_rapporto_from']
            deposito = self.dati_servizio['codice_deposito_from']
            fondo = self.dati_servizio['codice_fondo_from']
            tipo_contratto = self.dati_servizio['codice_tipocontratto_from']

        ######################
        # REINVESTI PROFITTI #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "REIPRO":
            if data['posizione_to_ex']:
                indice = int(data['posizione_to_ex'])
                rapporto = self.posizioni_to_ex[indice]['RAPPORTO']
                deposito = self.posizioni_to_ex[indice]['NUMCONTRATTO']
                fondo = self.posizioni_to_ex[indice]['CODFONDO']
                tipo_contratto = self.posizioni_to_ex[indice]['CODTC']

            if data['posizione_to_not']:
                indice = int(data['posizione_to_not'])
                rapporto = self.posizioni_to_not[indice]['RAPPORTO']
                deposito = self.posizioni_to_not[indice]['NUMCONTRATTO']
                fondo = self.posizioni_to_not[indice]['CODFONDO']
                tipo_contratto = self.posizioni_to_not[indice]['CODTC']

            valore = numero_rate_versate = 0
            frequenza = ""
            data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
            indice = int(data['posizione_to_ex'] or data['posizione_to_not'])
            rei_pro_perc_plus = data['rei_pro_perc_plus']

            if data['fine_servizio'] == "R":
                fine_servizio = "REVOCA"
            fine_servizio_anno = data['fine_servizio_anno']
            fine_servizio_mese = data['fine_servizio_mese']

        ###########################
        # RIBILANCIA INVESTIMENTO #
        ###########################
        elif self.dati_servizio['tipo_operazione'] == "RIBINV":
            rapporto = deposito = 0
            fondo = tipo_contratto = ""
            valore = numero_rate_versate = 0
            frequenza = ""
            data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
            fondo_gruppo_a = ""
            fondo_gruppo_b = ""

            if data['fine_servizio'] == "R":
                fine_servizio = "REVOCA"
            fine_servizio_anno = data['fine_servizio_anno']
            fine_servizio_mese = data['fine_servizio_mese']

            rib_inv_perc_scost = data['rib_inv_perc_scost']
            lista = []
            for obj_fondo in self.session['app_fondi_to_totali']:
                ind = "fondo_{}_{}_{}".format(obj_fondo['CODFONDO'], obj_fondo['RAPPORTO'], obj_fondo['NUMCONTRATTO'])
                peso = data.get(ind, 0)
                if peso:
                    lista.append("%s;%s;%s;%s;%s" % (
                        obj_fondo['RAPPORTO'], obj_fondo['NUMCONTRATTO'], obj_fondo['CODTC'],
                        obj_fondo['CODFONDO'], peso,
                    ))
            dettaglio_from = self.servizio.dettaglio_from
            dettaglio_to = "|".join(lista)

        ########################
        # RIMBORSO PROGRAMMATO #
        ########################
        elif self.dati_servizio['tipo_operazione'] == "RIMPRO":
            rapporto = self.dati_servizio['codice_rapporto_from']
            deposito = self.dati_servizio['codice_deposito_from']
            fondo = self.dati_servizio['codice_fondo_from']
            tipo_contratto = self.dati_servizio['codice_tipocontratto_from']

            iban = data.get('iban', "")

            valore = data['importo']
            if data['tipo_qta_movimento'] == "Q":
                valore = data['quote']
                tipo_qta_movimento = "QUOTE"

            frequenza = data['frequenza']
            data_decorrenza = data['data_decorrenza']
            if data['fine_servizio'] == "R":
                fine_servizio = "REVOCA"
            fine_servizio_anno = data['fine_servizio_anno']
            fine_servizio_mese = data['fine_servizio_mese']

        ######################
        # SWITCH PROGRAMMATO #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "SWIPRO":
            if data['posizione_to_ex']:
                indice = int(data['posizione_to_ex'])
                rapporto = self.posizioni_to_ex[indice]['RAPPORTO']
                deposito = self.posizioni_to_ex[indice]['NUMCONTRATTO']
                fondo = self.posizioni_to_ex[indice]['CODFONDO']
                tipo_contratto = self.posizioni_to_ex[indice]['CODTC']

            if data['posizione_to_not']:
                indice = int(data['posizione_to_not'])
                rapporto = self.posizioni_to_not[indice]['RAPPORTO']
                deposito = self.posizioni_to_not[indice]['NUMCONTRATTO']
                fondo = self.posizioni_to_not[indice]['CODFONDO']
                tipo_contratto = self.posizioni_to_not[indice]['CODTC']

            indice = int(data['posizione_to_ex'] or data['posizione_to_not'])

            valore = data['importo']
            if data['tipo_qta_movimento'] == "Q":
                valore = data['quote']
                tipo_qta_movimento = "QUOTE"

            frequenza = data['frequenza']
            data_decorrenza = data['data_decorrenza']
            if data['fine_servizio'] == "R":
                fine_servizio = "REVOCA"
            fine_servizio_anno = data['fine_servizio_anno']
            fine_servizio_mese = data['fine_servizio_mese']

            numero_rate_versate = data['numero_rate_versate']
            rad_rib_riduz_quota_a = data['rad_rib_riduz_quota_a']
            rad_rib_riduz_quota_b = data['rad_rib_riduz_quota_b']

        #####################
        # CHECK CAN OPERATE #
        #####################
        result = op_servizi.op_servizi_check_can_operate(
            self.dati_servizio['codice_fiscale'], self.dati_servizio['codice_cliente'],
            self.dati_servizio['tipo_operazione'],
            self.dati_servizio['codice_rapporto_from'], self.dati_servizio['codice_deposito_from'],
            self.dati_servizio['codice_fondo_from'], self.dati_servizio['codice_tipocontratto_from'],
            rapporto, deposito, fondo, tipo_contratto,
            tipo_qta_movimento, valore, numero_rate_versate,
            mifid, iban,
            frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            rei_pro_perc_plus, car_die_allocazione_a, car_die_allocazione_b,
            car_die_perc_var_a, car_die_perc_riall_ab, rad_rib_flag, rad_rib_riduz_quota_a,
            rad_rib_riduz_quota_b, rad_rib_nop_pianis, rib_inv_perc_scost, dettaglio_from,
            dettaglio_to, fondo_gruppo_a, fondo_gruppo_b
        )

        if not result['OK']:
            if result['CheckMIFID'] == "APPROPRIATEZZA":
                raise forms.ValidationError(result['ErrorMessage'])

            if result['CheckMIFID'] == "ADEGUATEZZA":
                raise forms.ValidationError(result['ErrorMessage'])

            if result['CheckMIFID'] == "NONE":
                raise forms.ValidationError(result['ErrorMessage'])

    def clean(self):
        data = self.cleaned_data

        #################
        # ATTIVA CEDOLA #
        #################
        if self.dati_servizio['tipo_operazione'] == "ATTCED":
            if not data['posizione_to'] and not data['iban']:
                raise forms.ValidationError(
                    "Selezionare 'Fondi AcomeA' o 'Conto Corrente'"
                )
            if data['posizione_to'] and data['iban']:
                raise forms.ValidationError(
                    "Selezionare 'Fondi AcomeA' o 'Conto Corrente'"
                )

            self.can_operate()

        ##############
        # CARPE DIEM #
        ##############
        elif self.dati_servizio['tipo_operazione'] == "CARDIE":
            if 'tcpic' not in data:
                raise forms.ValidationError("Numero di contratto non impostato")
            if not('car_die_allocazione_a' in data and 'car_die_allocazione_b' in data):
                raise forms.ValidationError("Impostare percentuali di allocazione A e B")
            if not('car_die_gruppo_a' in data):
                raise forms.ValidationError("Selezionare Fondi Grupppo A e Gruppo B")
            if (
                'car_die_allocazione_a' in data and 'car_die_allocazione_b' in data and
                int(data['car_die_allocazione_a']) + int(data['car_die_allocazione_b']) != 100):
                raise forms.ValidationError("La somma delle percentuali di allocazione A/B deve essere sempre 100%")

            self.can_operate()

        #####################
        # RADDOPPIA RIBASSO #
        #####################
        if self.dati_servizio['tipo_operazione'] == "RADRIB":

            if data['fine_servizio'] == 'R':
                if data['fine_servizio_anno'] != '0' or data['fine_servizio_mese'] != '0':
                    raise forms.ValidationError(_("Per fine servizio Revoca non inserire mese e anno"))
            if data['fine_servizio'] == "D":
                if data['fine_servizio_anno'] == '0' or data['fine_servizio_mese'] == '0':
                    raise forms.ValidationError(_("Inserire Mese e Anno di fine servizio"))

            if data['rad_rib_riduz_quota_a'] == '0' and data['rad_rib_riduz_quota_b'] == '0':
                raise forms.ValidationError(_("Selezionare percentuale per Ipotesi A o B"))
            if data['rad_rib_riduz_quota_a'] != '0' and data['rad_rib_riduz_quota_b'] != '0':
                raise forms.ValidationError(_("Selezionare percentuale per Ipotesi A o B"))
            if data.get('posizioni_pianis_rid') == "" and data.get('posizioni_pianis_swipro') == "":
                raise forms.ValidationError(
                    _('Seleziona dove attivare il servizio "Raddoppia sul Ribasso"')
                )

            self.can_operate()

        ######################
        # REINVESTI PROFITTI #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "REIPRO":
            if data['rei_pro_perc_plus'] == 0:
                raise forms.ValidationError("Selezionare '% di Plusvalenza'")

            if data['fine_servizio'] == 'R':
                if data['fine_servizio_anno'] != '0' or data['fine_servizio_mese'] != '0':
                    raise forms.ValidationError(_("Per fine servizio Revoca non inserire mese e anno"))

            if data['fine_servizio'] == "D":
                if data['fine_servizio_anno'] == '0' or data['fine_servizio_mese'] == '0':
                    raise forms.ValidationError(_("Inserire Mese e Anno di fine servizio"))

            if not data['posizione_to_ex'] and not data['posizione_to_not']:
                raise forms.ValidationError("Selezionare 'Posizioni In Essere' o 'Nuova posizione'")
            if data['posizione_to_ex'] and data['posizione_to_not']:
                raise forms.ValidationError("Selezionare o 'Posizioni In Essere' o 'Nuova posizione'")

            self.can_operate()

        ###########################
        # RIBILANCIA INVESTIMENTO #
        ###########################
        elif self.dati_servizio['tipo_operazione'] == "RIBINV":
            if data['fine_servizio'] == 'R':
                if data['fine_servizio_anno'] != '0' or data['fine_servizio_mese'] != '0':
                    raise forms.ValidationError(_("Per fine servizio Revoca non inserire mese e anno"))

            if data['fine_servizio'] == "D":
                if data['fine_servizio_anno'] == '0' or data['fine_servizio_mese'] == '0':
                    raise forms.ValidationError(_("Inserire Mese e Anno di fine servizio"))

            for fondo in self.fondi_totali_to:
                ind = "fondo_{}_{}_{}".format(fondo['CODFONDO'], fondo['RAPPORTO'], fondo['NUMCONTRATTO'])
                if not ind in data:
                    raise forms.ValidationError("Tutti i pesi dei fondi devono essere compresi tra 0 e 100")
            fund_weights = [v for k, v in data.iteritems() if k.startswith("fondo_")]
            if 100 in fund_weights:
                raise forms.ValidationError(
                    "Inserire il peso per almeno due fondi. La sommatoria deve essere uguale a 100"
                )
            self.can_operate()

        ########################
        # RIMBORSO PROGRAMMATO #
        ########################
        elif self.dati_servizio['tipo_operazione'] == "RIMPRO":
            if not 'data_decorrenza' in data:
                raise forms.ValidationError(_("Inserire una Data Decorrenza valida"))

            if not 'tipo_qta_movimento' in data:
                raise forms.ValidationError("Selezionare tipo quantità movimento")

            if data['tipo_qta_movimento'] == "I":
                if not data['importo']:
                    raise forms.ValidationError(_("Inserire un importo valido"))
                if data['importo'] and data['importo'] <= 0:
                    raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
                if data['quote']:
                    raise forms.ValidationError(_("Switch per importo non prevede la presenza di quote"))

            if data['tipo_qta_movimento'] == "Q":
                if not data['quote']:
                    raise forms.ValidationError(_("Inserire un numero quote valido"))
                if data['quote'] <= 0:
                    raise forms.ValidationError(_("Il numero di quote deve essere maggiore di zero"))
                if data['importo']:
                    raise forms.ValidationError(_("Switch per quote non prevede la presenza di importi"))

            if data['fine_servizio'] == 'R':
                if data['fine_servizio_anno'] != '0' or data['fine_servizio_mese'] != '0':
                    raise forms.ValidationError(_("Per fine servizio Revoca non inserire mese e anno"))

            if data['fine_servizio'] == "D":
                if data['fine_servizio_anno'] == '0' or data['fine_servizio_mese'] == '0':
                    raise forms.ValidationError(_("Inserire Mese e Anno di fine servizio"))

            self.can_operate()

        ######################
        # SWITCH PROGRAMMATO #
        ######################
        elif self.dati_servizio['tipo_operazione'] == "SWIPRO":
            if not data['posizione_to_ex'] and not data['posizione_to_not']:
                raise forms.ValidationError("Selezionare 'Posizioni In Essere' o 'Nuova posizione'")
            if data['posizione_to_ex'] and data['posizione_to_not']:
                raise forms.ValidationError("Selezionare o 'Posizioni In Essere' o 'Nuova posizione'")

            if data['posizione_to_ex']:
                tipo_piano = 'PIC'

            if data['posizione_to_not']:
                tipo_piano = data['tipo_piano_not']
                data['numero_rate_versate'] = 0

            if not 'data_decorrenza' in data:
                raise forms.ValidationError(_("Inserire una Data Decorrenza valida"))

            if tipo_piano == 'PIC':
                if not 'numero_rate_versate' in data:
                    data['numero_rate_versate'] = 0

            if tipo_piano == 'PAC':
                if not 'numero_rate_versate' in data:
                    raise forms.ValidationError(_("Inserire un numero rate versate valido"))
                if data['numero_rate_versate'] <= 0:
                    raise forms.ValidationError(_("Il numero di rate deve essere maggiore di zero"))
                if data['tipo_qta_movimento'] == "Q":
                    raise forms.ValidationError(_("Switch per Quote non previsto per piani PAC"))

            if not 'tipo_qta_movimento' in data:
                raise forms.ValidationError("Selezionare tipo quantità movimento")

            if data['tipo_qta_movimento'] == "I":
                if not data['importo']:
                    raise forms.ValidationError(_("Inserire un importo valido"))
                if data['importo'] and data['importo'] <= 0:
                    raise forms.ValidationError(_("Inserire un importo maggiore di zero"))
                if data['quote']:
                    raise forms.ValidationError(_("Switch per importo non prevede la presenza di quote"))

            if data['tipo_qta_movimento'] == "Q":
                if not data['quote']:
                    raise forms.ValidationError(_("Inserire un numero quote valido"))
                if data['quote'] <= 0:
                    raise forms.ValidationError(_("Il numero di quote deve essere maggiore di zero"))
                if data['importo']:
                    raise forms.ValidationError(_("Switch per quote non prevede la presenza di importi"))

            if data['fine_servizio'] == 'R':
                if data['fine_servizio_anno'] != '0' or data['fine_servizio_mese'] != '0':
                    raise forms.ValidationError(_("Per fine servizio Revoca non inserire mese e anno"))

            if data['fine_servizio'] == "D":
                if data['fine_servizio_anno'] == '0' or data['fine_servizio_mese'] == '0':
                    raise forms.ValidationError(_("Inserire Mese e Anno di fine servizio"))

            if data['rad_rib_flag']:
                if data['rad_rib_riduz_quota_a'] == '0' and data['rad_rib_riduz_quota_b'] == '0':
                    raise forms.ValidationError(_("Selezionare percentuale per Ipotesi A o B"))
                if data['rad_rib_riduz_quota_a'] != '0' and data['rad_rib_riduz_quota_b'] != '0':
                    raise forms.ValidationError(_("Selezionare percentuale per Ipotesi A o B"))
            else:
                if data['rad_rib_riduz_quota_a'] != '0' or data['rad_rib_riduz_quota_b'] != '0':
                    raise forms.ValidationError(_("Opzione 'Raddoppia sul Ribasso' non attivata"))

            if 'mifid' in data:
                if data['mifid']:
                    self.session['data_operazione']['mifid'] = True

            self.can_operate()
        return data



class ServizioRiepilogoForm(forms.Form):
    def __init__(self, *args, **kwargs):
        session = kwargs.pop('session', None)
        self.dati_servizio = session['app_dati_servizio']
        super(ServizioRiepilogoForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = self.cleaned_data
        return data
