# coding: utf-8
"""
soap module for acomea.app
"""

import locale
import logging
import datetime
import socket

from pysimplesoap.client import SoapClient

from django.conf import settings

log = logging.getLogger("acomea.areariservata")


# ecommerce
class OpServizi:
    """
    SOAP_AR: http://93.57.3.196:8282/service.asmx?wsdl
    """
    def __init__(self):
        log.debug("OpServizi init")
        self._client = None

    @property
    def client(self):
        if settings.SOAP_RESERVED_URL and not self._client:
            try:
                self._client = SoapClient(wsdl=settings.SOAP_RESERVED_URL, trace=False)
            except socket.timeout as err:
                log.warning("OpServizi timeout Exception %s" % str(err))
                self._client = None
            except Exception, exc:
                log.warning("OpServizi Exception %s" % str(exc))
                raise
        return self._client

    def is_area_active(self):
        """
        Restituisce lo stato dell'Area Riservata

        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive

        RETURN:
        <AreaRiservata_IsActive_OpOnlineRimborsoResult>boolean</AreaRiservata_IsActive_OpOnlineRimborsoResult>
        """
        data = self.client.AreaRiservata_IsActive()
        return data['AreaRiservata_IsActiveResult']

    def get_cliente(self, cod_fiscale):
        """
        Restituisce una struttura coi dati principali di un cliente dato il suo codice fiscale,
        o None se il cliente non esiste.

        http://93.57.3.196:8282/service.asmx?op=GetCliente

        SEND
        <cf>string</cf>

        RETURN
        <IsIntestatario>boolean</IsIntestatario>
        <IsCoIntestatario>boolean</IsCoIntestatario>
        <IsDead>boolean</IsDead>
        <IsPF>boolean</IsPF>
        <Found>boolean</Found>
        <Patrimonio>double</Patrimonio>
        <DtNasc>string</DtNasc>
        <CodCliFC>string</CodCliFC>
        <Nome>string</Nome>
        <Cognome>string</Cognome>
        <Mail>string</Mail>
        <CODFISC>string</CODFISC>
        <ProfiloAppropriatezza>string</ProfiloAppropriatezza>
        <ProfiloAdeguatezza>string</ProfiloAdeguatezza>
        <TipoPartecipante>string</TipoPartecipante>
        <ClasseFondi>A1 or A2 or A1A2</ClasseFondi>
        """
        data = self.client.GetCliente(cod_fiscale)['GetClienteResult']
        if data['CODFISC']:
            return data
        else:
            return None

    def check_cliente(self, cod_cliente, cod_fiscale, data_nascita):
        """
        Controlla se un cliente esistente è attivo.

        http://93.57.3.196:8282/service.asmx?op=CheckCliente

        <codcli>long</codcli>
        <cf>string</cf>
        <dtnasc>string</dtnasc>

        RETURN
        <CheckClienteResult>boolean</CheckClienteResult>
        """
        data = self.client.CheckCliente(cod_cliente, cod_fiscale, _format_data(data_nascita))
        return data['CheckClienteResult']

    def op_servizi_datagrid(self, cod_cli, tipo_partecipazione):
        """
        Restituisce la lista delle partecipazioni

        http://93.57.3.196:8282/service.asmx?op=OpServizi_DataGrid

        SEND
        <CodCliente>int</CodCliente>
        <tp>INTESTATARIO or COINTESTATARIO or DELEGATO</tp>

        RETURN
        <PosizioniSource>
            <CSOTDEP>long</CSOTDEP>
            <RAPPORTO>long</RAPPORTO>
            <NUMCONTRATTO>string</NUMCONTRATTO>
            <CODFONDO>string</CODFONDO>
            <DESCRFONDO>string</DESCRFONDO>
            <CODTC>string</CODTC>
            <DESCRTC>string</DESCRTC>
            <TIPOPIANO>string</TIPOPIANO>
            <NUMRATEPAC>string</NUMRATEPAC>
            <QUOTE>decimal</QUOTE>
            <PATRIMONIO>decimal</PATRIMONIO>
            <IMPORTORATA>decimal</IMPORTORATA>
        </PosizioniSource>
        """
        posizioni = self.client.OpServizi_DataGrid(cod_cli, tipo_partecipazione)['OpServizi_DataGridResult']
        lista_posizioni = []
        for posizione in posizioni:
            lista_posizioni.append(posizione['PosizioniSource'])
        return lista_posizioni

    def qst_must_compile_ar_by_rapp(self, cod_fiscale, cod_rapporto, cod_fondo):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_MustCompileARByRap

        RETURN
        <cf>string</cf>
        <cod_rapp>long</cod_rapp>
        <cod_fon>string</cod_fon>
        """
        # TODO
        return self.client.QST_MustCompileARByRap(cod_fiscale, cod_rapporto, cod_fondo)['QST_MustCompileARByRapResult']

    def qst_must_compile_ar_by_sott(self, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_MustCompileARBySot

        RETURN
        <cf>string</cf>
        All'attivazione del profilo dispositivo occorre chiamare questo metodo
        """
        # TODO
        return self.client.QST_MustCompileARBySot(cod_fiscale)['QST_MustCompileARBySotResult']

    def qst_is_compiled(self, questionario, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_IsCompiled

        RETURN
        <_tq>ANTIRICICLAGGIO or APPROPRIATEZZA or ADEGUATEZZA</_tq>
        <_cf>string</_cf>
        """
        # TODO
        return self.client.QST_IsCompiled(questionario, cod_fiscale)['QST_IsCompiledResult']

    def qst_is_compiled_reset(self, questionario, cod_fiscale):
        """
        Resetta questionari, utilizzabile sui seguenti codici fiscali:
        http://93.57.3.196:8282/service.asmx?op=QST_IsCompiledReset

        RETURN
        NGRNRE21A66G337Q;SLLGPP36C30F151F;CTLPQL54B27E716D;MRNLRD64H16G843J;
        CCCPLA73M03C933F;LDAGPP39P11F205X;VTLFBA66P10F205J;CLPMPA31R11L433I
        """
        # TODO
        return self.client.QST_IsCompiledReset(questionario, cod_fiscale)

    def op_servizi_check_can_operate(
            self, cod_fisc, cod_cli, tipo_servizio, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento, num_rate_versate,
            check_manleva, iban, frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            reipro_perc_plus, cardie_allocazione_a, cardie_allocazione_b, cardie_perc_val_a, cardie_perc_riall_ab,
            radrib_flag, radrib_riduz_quota_a, radrib_riduz_quota_b, radrib_nop_pianis,
            ribinv_perc_scost, dettaglio_from, dettaglio_to, fondo_gruppo_a, fondo_gruppo_b
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_CheckCanOperate

        SEND
        <cf>string</cf>
        <c_sot>long</c_sot>
        <tipoServizio>SWIPRO or RIMPRO or ATTCED or REIPRO or CARDIE or RIBINV or RADRIB</tipoServizio>
        <codRappFrom>string</codRappFrom>
        <codDepFrom>string</codDepFrom>
        <codFonFrom>string</codFonFrom>
        <tcFrom>string</tcFrom>
        <codRappTo>string</codRappTo>
        <codDepTo>string</codDepTo>
        <codFonTo>string</codFonTo>
        <tcTo>string</tcTo>
        <TipoQtaMovimento>QUOTE or IMPORTO or TOTALE</TipoQtaMovimento>
        <ValoreQtaMovimento>decimal</ValoreQtaMovimento>
        <NumRateVersate>decimal</NumRateVersate>
        <CheckManLeva>string</CheckManLeva>
        <IBAN>string</IBAN>
        <Frequenza>string</Frequenza>
        <dtdecorrenza>string</dtdecorrenza>
        <FineServizio>REVOCA or DATA</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>
        <REIPROPercPlus>short</REIPROPercPlus>
        <CARDIEAllocazioneA>short</CARDIEAllocazioneA>
        <CARDIEAllocazioneB>short</CARDIEAllocazioneB>
        <CARDIEPercValA>short</CARDIEPercValA>
        <CARDIEPercRiallB>short</CARDIEPercRiallB>
        <RADRIB_flg>string</RADRIB_flg>
        <RADRIB_RiduzQuotaA>short</RADRIB_RiduzQuotaA>
        <RADRIB_RiduzQuotaB>short</RADRIB_RiduzQuotaB>
        <RADRIBNopPianiS>long</RADRIBNopPianiS>
        <RIBINVpercScost>short</RIBINVpercScost>
        <DettaglioFrom>string</DettaglioFrom>
        <DettaglioTo>string</DettaglioTo>
        <CARDIECodFonA>string</CARDIECodFonA>
        <CARDIECodFonB>string</CARDIECodFonB>

        RETURN
        <OpServizi_CheckCanOperateResult>
            <OK>boolean</OK>
            <CheckMIFID>APPROPRIATEZZA or ADEGUATEZZA or NONE</CheckMIFID>
            <ErrorMessage>string</ErrorMessage>
            <Summary>string</Summary>
        </OpServizi_CheckCanOperateResult>
        """

        # print (
        #     cod_fisc, cod_cli, tipo_servizio, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
        #     cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento, num_rate_versate,
        #     check_manleva, iban, frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
        #     reipro_perc_plus, cardie_allocazione_a, cardie_allocazione_b, cardie_perc_val_a, cardie_perc_riall_ab,
        #     radrib_flag, radrib_riduz_quota_a, radrib_riduz_quota_b, radrib_nop_pianis,
        #     ribinv_perc_scost, dettaglio_from, dettaglio_to, fondo_gruppo_a, fondo_gruppo_b
        # )

        return self.client.OpServizi_CheckCanOperate(
            cod_fisc, cod_cli, tipo_servizio, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento, num_rate_versate,
            check_manleva, iban, frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            reipro_perc_plus, cardie_allocazione_a, cardie_allocazione_b, cardie_perc_val_a, cardie_perc_riall_ab,
            radrib_flag, radrib_riduz_quota_a, radrib_riduz_quota_b, radrib_nop_pianis,
            ribinv_perc_scost, dettaglio_from, dettaglio_to, fondo_gruppo_a, fondo_gruppo_b
        )['OpServizi_CheckCanOperateResult']

    def op_servizi_get_elenco_switch(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoSwitch
        SEND
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>

        RETURN
        <PosizioniTarget>
            <RAPPORTO>long</RAPPORTO>
            <NUMCONTRATTO>string</NUMCONTRATTO>
            <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
            <CODFONDO>string</CODFONDO>
            <DESCRFONDO>string</DESCRFONDO>
            <CODTC>string</CODTC>
            <DESCRTC>string</DESCRTC>
            <TIPOPIANO>string</TIPOPIANO>
            <NUMRATEPAC>string</NUMRATEPAC>
            <IMPORTORATA>decimal</IMPORTORATA>
        </PosizioniTarget>
        """

        posizioni = self.client.OpServizi_GetElencoSwitch(
            cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc
        )['OpServizi_GetElencoSwitchResult']

        lista_posizioni = []
        for posizione in posizioni:
            lista_posizioni.append(posizione['PosizioniTarget'])
        return lista_posizioni

    def op_servizi_get_elenco_switch_ex(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoSwitchEX

        SEND
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>

        RETURN
        <PosizioniTarget>
            <RAPPORTO>long</RAPPORTO>
            <NUMCONTRATTO>string</NUMCONTRATTO>
            <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
            <CODFONDO>string</CODFONDO>
            <DESCRFONDO>string</DESCRFONDO>
            <CODTC>string</CODTC>
            <DESCRTC>string</DESCRTC>
            <TIPOPIANO>string</TIPOPIANO>
            <NUMRATEPAC>string</NUMRATEPAC>
            <IMPORTORATA>decimal</IMPORTORATA>
        </PosizioniTarget>
        """

        posizioni_essere = self.client.OpServizi_GetElencoSwitchEX(
            cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc
        )['OpServizi_GetElencoSwitchEXResult']

        lista_posizioni = []
        for posizione in posizioni_essere:
            lista_posizioni.append(posizione['PosizioniTarget'])
        return lista_posizioni

    def op_servizi_get_elenco_switch_not(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoSwitchNOT

        SEND
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>

        RETURN
        <PosizioniTarget>
            <RAPPORTO>long</RAPPORTO>
            <NUMCONTRATTO>string</NUMCONTRATTO>
            <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
            <CODFONDO>string</CODFONDO>
            <DESCRFONDO>string</DESCRFONDO>
            <CODTC>string</CODTC>
            <DESCRTC>string</DESCRTC>
            <TIPOPIANO>string</TIPOPIANO>
            <NUMRATEPAC>string</NUMRATEPAC>
            <IMPORTORATA>decimal</IMPORTORATA>
        </PosizioniTarget>
        """
        posizioni_nuove = self.client.OpServizi_GetElencoSwitchNOT(
            cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc
        )['OpServizi_GetElencoSwitchNOTResult']
        lista_posizioni = []
        for posizione in posizioni_nuove:
            lista_posizioni.append(posizione['PosizioniTarget'])
        return lista_posizioni

    def op_servizi_get_elenco_servizi_pianis(self, tipo, cod_rapp):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoServiziPianis

        SEND
        <Tipo>string</Tipo>
        <cod_rapp>long</cod_rapp>

        RETURN
        <PosizioniTargetPianis>
            <ID>long</ID>
            <QUOTE>decimal</QUOTE>
            <IMPORTO>decimal</IMPORTO>
            <RAPPORTO>long</RAPPORTO>
            <NUMCONTRATTO>string</NUMCONTRATTO>
            <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
            <CODFONDO>string</CODFONDO>
            <DESCRFONDO>string</DESCRFONDO>
            <CODTC>string</CODTC>
            <DESCRTC>string</DESCRTC>
            <TIPOPIANO>string</TIPOPIANO>
        </PosizioniTargetPianis>
        """
        posizioni_pianis = self.client.OpServizi_GetElencoServiziPianis(
            tipo, cod_rapp
        )['OpServizi_GetElencoServiziPianisResult']
        lista_posizioni = []
        for posizione in posizioni_pianis:
            lista_posizioni.append(posizione['PosizioniTargetPianis'])
        return lista_posizioni

    def op_servizi_add_switch_programmato(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, quote, importo, num_rate_piano, importo_rata, num_rate_versate, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc, frequenza,
            data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            radrib_flag, radrib_riduz_quota_a, radrib_riduz_quota_b
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDSwitchProgrammato

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <quote>decimal</quote>
        <importo>decimal</importo>
        <numratepiano>long</numratepiano>
        <importorata>decimal</importorata>
        <numrateversate>decimal</numrateversate>
        <simulazione>string</simulazione>

        <switchcodrapp>long</switchcodrapp>
        <switchcoddep>string</switchcoddep>
        <switchcodfon>string</switchcodfon>
        <switchtc>string</switchtc>
        <Frequenza>string</Frequenza>

        <dtdecorrenza>string</dtdecorrenza>
        <FineServizio>Revoca or Data</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>

        <RADRIB_flg>string</RADRIB_flg>
        <RADRIB_RiduzQuotaA>short</RADRIB_RiduzQuotaA>
        <RADRIB_RiduzQuotaB>short</RADRIB_RiduzQuotaB>

        RETURN
        <OpServizi_ADDSwitchProgrammatoResult />
        """

        return self.client.OpServizi_ADDSwitchProgrammato(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, quote, importo, num_rate_piano, importo_rata, num_rate_versate, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc, frequenza,
            data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            radrib_flag, radrib_riduz_quota_a, radrib_riduz_quota_b
        )['OpServizi_ADDSwitchProgrammatoResult']

    def op_servizi_add_rimborso_programmato(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, quote, importo, num_rate_piano, importo_rata, num_rate_versate, simulazione,
            iban, bic, frequenza,
            data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDRimborsoProgrammato

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <quote>decimal</quote>
        <importo>decimal</importo>
        <numratepiano>long</numratepiano>
        <importorata>decimal</importorata>
        <numrateversate>decimal</numrateversate>
        <simulazione>string</simulazione>

        <IBAN>string</IBAN>
        <BIC>string</BIC>
        <Frequenza>string</Frequenza>
        <dtdecorrenza>string</dtdecorrenza>
        <FineServizio>Revoca or Data</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>

        RETURN
        <OpServizi_ADDRimborsoProgrammatoResult />
        """

        return self.client.OpServizi_ADDRimborsoProgrammato(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, quote, importo, num_rate_piano, importo_rata, num_rate_versate, simulazione,
            iban, bic, frequenza,
            data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese
        )['OpServizi_ADDRimborsoProgrammatoResult']

    def op_servizi_add_attiva_cedola(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            iban, bic
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDAttivaCedola

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <numratepiano>long</numratepiano>
        <importorata>decimal</importorata>
        <simulazione>string</simulazione>

        <switchcodrapp>long</switchcodrapp>
        <switchcoddep>string</switchcoddep>
        <switchcodfon>string</switchcodfon>
        <switchtc>string</switchtc>

        <IBAN>string</IBAN>
        <BIC>string</BIC>

        RETURN
        <OpServizi_ADDAttivaCedolaResult />
        """

        return self.client.OpServizi_ADDAttivaCedola(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            iban, bic
        )['OpServizi_ADDAttivaCedolaResult']

    def op_servizi_add_reinvesti_profitti(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            perc_plus
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDReinvestiProfitti

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <numratepiano>long</numratepiano>
        <importorata>decimal</importorata>
        <simulazione>string</simulazione>

        <switchcodrapp>long</switchcodrapp>
        <switchcoddep>string</switchcoddep>
        <switchcodfon>string</switchcodfon>
        <switchtc>string</switchtc>

        <Frequenza>string</Frequenza>
        <dtdecorrenza>string</dtdecorrenza>
        <FineServizio>Revoca or Data</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>

        <PercPlus>short</PercPlus>

        RETURN
        <OpServizi_ADDReinvestiProfittiResult />
        """

        return self.client.OpServizi_ADDReinvestiProfitti(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            frequenza, data_decorrenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            perc_plus
        )['OpServizi_ADDReinvestiProfittiResult']

    def op_servizi_add_carpe_diem(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            perc_allocazione_a, perc_allocazione_b, per_var_a, perc_riall_b,
            dettaglio_from
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDCarpeDiem

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <numratepiano>long</numratepiano>
        <importorata>decimal</importorata>
        <simulazione>string</simulazione>

        <switchcodrapp>long</switchcodrapp>
        <switchcoddep>string</switchcoddep>
        <switchcodfon>string</switchcodfon>
        <switchtc>string</switchtc>

        <PercAllocazioneA>decimal</PercAllocazioneA>
        <PercAllocazioneB>decimal</PercAllocazioneB>
        <PercVarA>decimal</PercVarA>
        <PercRiallAB>decimal</PercRiallAB>
        <DettaglioFrom>string</DettaglioFrom>

        RETURN
        <OpServizi_ADDCarpeDiemResult />
        """

        return self.client.OpServizi_ADDCarpeDiem(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, num_rate_piano, importo_rata, simulazione,
            switch_cod_rapp, switch_cod_dep, switch_cod_fon, switch_tc,
            perc_allocazione_a, perc_allocazione_b, per_var_a, perc_riall_b, dettaglio_from
        )['OpServizi_ADDCarpeDiemResult']

    def op_servizi_add_ribilancia_investimento(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, simulazione, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            perc_scost, dettaglio_from, dettaglio_to
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDRibilanciaInvestimento

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>
        <mifid>string</mifid>
        <simulazione>string</simulazione>
        <FineServizio>Revoca or Data</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>
        <PercScost>short</PercScost>
        <DettaglioFrom>string</DettaglioFrom>
        <DettaglioTo>string</DettaglioTo>

        RETURN
        <OpServizi_ADDRibilanciaInvestimentoResult />
        """

        return self.client.OpServizi_ADDRibilanciaInvestimento(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, simulazione, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            perc_scost, dettaglio_from, dettaglio_to
        )['OpServizi_ADDRibilanciaInvestimentoResult']

    def op_servizi_add_raddoppia_ribasso(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, simulazione, frequenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            radrib_riduz_quota_a, radrib_riduz_quota_b, id_pianis
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDRaddoppiaRibassoPianis

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <mifid>string</mifid>
        <simulazione>string</simulazione>
        <Frequenza>string</Frequenza>
        <FineServizio>Revoca or Data</FineServizio>
        <FineServizioAnno>short</FineServizioAnno>
        <FineServizioMese>short</FineServizioMese>
        <RADRIB_RiduzQuotaA>short</RADRIB_RiduzQuotaA>
        <RADRIB_RiduzQuotaB>short</RADRIB_RiduzQuotaB>
        <IDPianis>long</IDPianis>

        RETURN
        <OpServizi_ADDRaddoppiaRibassoPianisResult />
        """

        return self.client.OpServizi_ADDRaddoppiaRibassoPianis(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            mifid, simulazione, frequenza, fine_servizio, fine_servizio_anno, fine_servizio_mese,
            radrib_riduz_quota_a, radrib_riduz_quota_b, id_pianis
        )['OpServizi_ADDRaddoppiaRibassoPianisResult']

    def op_servizi_add_revoca(
            self, cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            simulazione, tipo_servizio, id_servizio_as400
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDRevoca

        SEND
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>

        <simulazione>string</simulazione>
        <TipoServizio>SWIPRO or RIMPRO or ATTCED or REIPRO or CARDIE or RIBINV or RADRIB</TipoServizio>
        <IDServizioAS400>long</IDServizioAS400>

        RETURN
        <OpServizi_ADDRevocaResult />
        """

        return self.client.OpServizi_ADDRevoca(
            cod_cli, cod_rapp, cod_dep, cod_fon, tipo_contratto, cognome, nome, cod_fisc,
            simulazione, tipo_servizio, id_servizio_as400
        )['OpServizi_ADDRevocaResult']

    def op_servizi_annulla_servizio_web(self, nop, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=AnnullaServizioWEB

        SEND
        <_nop>long</_nop>
        <_cf>string</_cf>

        RETURN
        <AnnullaServizioWEBResult>boolean</AnnullaServizioWEBResult>
        """
        return self.client.AnnullaServizioWEB(nop, cod_fiscale)['AnnullaServizioWEBResult']

    def op_servizi_revoca_servizio_attivo(self, nop, cod_fiscale, simulazione, tipo_servizio):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ADDRevoca1

        SEND
        <codfisc>string</codfisc>
        <simulazione>string</simulazione>
        <TipoServizio>SWIPRO or RIMPRO or ATTCED or REIPRO or CARDIE or RIBINV or RADRIB</TipoServizio>
        <IDServizioAS400>long</IDServizioAS400>

        RETURN
        <OpServizi_ADDRevoca1Result>long</OpServizi_ADDRevoca1Result>
        """
        return self.client.OpServizi_ADDRevoca1(
            cod_fiscale, simulazione, tipo_servizio, nop
        )['OpServizi_ADDRevoca1Result']

    def op_servizi_get_servizi_in_corso_web(self, cod_fiscale):
        """
        SEND
        <cf>string</cf>

        RETURN
        <ServiziWEB>
          <ID>long</ID>
          <DATI>string</DATI>
          <TIPO>string</TIPO>
          <RAPPORTO>long</RAPPORTO>
          <NUMCONTRATTO>string</NUMCONTRATTO>
          <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
          <CODFONDO>string</CODFONDO>
          <DESCRFONDO>string</DESCRFONDO>
          <DATAOPERAZIONE>dateTime</DATAOPERAZIONE>
          <CODFISC>string</CODFISC>
        </ServiziWEB>
        """
        servizi = self.client.OpServizi_GetServiziInCorsoWEB(cod_fiscale)['OpServizi_GetServiziInCorsoWEBResult']
        lista_servizi = []
        for servizio in servizi:
            lista_servizi.append(servizio['ServiziWEB'])
        return lista_servizi

    def op_servizi_get_servizi_attivi(self, cod_fiscale):
        """
        SEND
        <cf>string</cf>

        RETURN
        <ServiziWEB>
          <ID>long</ID>
          <DATI>string</DATI>
          <TIPO>string</TIPO>
          <RAPPORTO>long</RAPPORTO>
          <NUMCONTRATTO>string</NUMCONTRATTO>
          <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
          <CODFONDO>string</CODFONDO>
          <DESCRFONDO>string</DESCRFONDO>
          <DATAOPERAZIONE>dateTime</DATAOPERAZIONE>
          <CODFISC>string</CODFISC>
          <REVOCA>string</REVOCA>
        </ServiziWEB>
        """
        servizi = self.client.OpServizi_GetServiziAttivi(cod_fiscale)['OpServizi_GetServiziAttiviResult']
        lista_servizi = []
        for servizio in servizi:
            lista_servizi.append(servizio['ServiziWEB'])
        return lista_servizi

    def op_servizi_controlli_posizione_from(self, tipo_servizio, dettaglio_from):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_ControlliPosizioneFrom
        SEND
        <tipoServizio>SWIPRO or RIMPRO or ATTCED or REIPRO or CARDIE or RIBINV or RADRIB</tipoServizio>
        <Dettaglio>string</Dettaglio>

        RETURN
        <OK>boolean</OK>
        <CheckMIFID>APPROPRIATEZZA or ADEGUATEZZA or NONE</CheckMIFID>
        <ErrorMessage>string</ErrorMessage>
        <Summary>string</Summary>
        """
        return self.client.OpServizi_ControlliPosizioneFrom(
            tipo_servizio, dettaglio_from
        )['OpServizi_ControlliPosizioneFromResult']

    def op_servizi_get_elenco_fondi_cardie(self, cod_fondo, gruppo):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoFondiCARDIE
        SEND
          <_classe>string</_classe>
          <_gruppo>string</_gruppo>

        RETURN
        <OpServizi_GetElencoFondiCARDIEResult>
            <ElencoFondi>
                <CODFON>string</CODFON>
                <DESCRFON>string</DESCRFON>
                <CLASSE>string</CLASSE>
            </ElencoFondi>
        </OpServizi_GetElencoFondiCARDIEResult>
        """
        fondi = self.client.OpServizi_GetElencoFondiCARDIE(cod_fondo, gruppo)['OpServizi_GetElencoFondiCARDIEResult']
        lista_fondi = []
        for fondo in fondi:
            lista_fondi.append(fondo['ElencoFondi'])
        return lista_fondi

    def op_servizi_get_elenco_ribinv_ex(self, cod_cliente, dettaglio_from):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoRIBINVEX

        SEND
        <codCliente>int</codCliente>
        <DettaglioFrom>string</DettaglioFrom>

        RETURN
        <OpServizi_GetElencoRIBINVEXResult>
            <PosizioniTargetRIBINV>
                <RAPPORTO>long</RAPPORTO>
                <NUMCONTRATTO>string</NUMCONTRATTO>
                <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
                <CODFONDO>string</CODFONDO>
                <DESCRFONDO>string</DESCRFONDO>
                <CODTC>string</CODTC>
                <DESCRTC>string</DESCRTC>
                <TIPOPIANO>string</TIPOPIANO>
                <NUMRATEPAC>string</NUMRATEPAC>
                <IMPORTORATA>decimal</IMPORTORATA>
                <PESO>decimal</PESO>
                <CTV>decimal</CTV>
            </PosizioniTargetRIBINV>
        </OpServizi_GetElencoRIBINVEXResult>
        """
        fondi = self.client.OpServizi_GetElencoRIBINVEX(
            cod_cliente, dettaglio_from
        )['OpServizi_GetElencoRIBINVEXResult']
        lista_fondi = []
        for fondo in fondi:
            lista_fondi.append(fondo['PosizioniTargetRIBINV'])
        return lista_fondi

    def op_servizi_get_elenco_ribinv_not_ex(self, cod_cliente, dettaglio_from):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoRIBINVNOTEX

        SEND
        <codCliente>int</codCliente>
        <DettaglioFrom>string</DettaglioFrom>

        RETURN
        <OpServizi_GetElencoRIBINVNOTEXResult>
            <PosizioniTargetRIBINV>
                <RAPPORTO>long</RAPPORTO>
                <NUMCONTRATTO>string</NUMCONTRATTO>
                <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
                <CODFONDO>string</CODFONDO>
                <DESCRFONDO>string</DESCRFONDO>
                <CODTC>string</CODTC>
                <DESCRTC>string</DESCRTC>
                <TIPOPIANO>string</TIPOPIANO>
                <NUMRATEPAC>string</NUMRATEPAC>
                <IMPORTORATA>decimal</IMPORTORATA>
                <PESO>decimal</PESO>
                <CTV>decimal</CTV>
            </PosizioniTargetRIBINV>
        </OpServizi_GetElencoRIBINVEXResult>
        """
        fondi = self.client.OpServizi_GetElencoRIBINVNOTEX(
            cod_cliente, dettaglio_from
        )['OpServizi_GetElencoRIBINVNOTEXResult']
        lista_fondi = []
        for fondo in fondi:
            lista_fondi.append(fondo['PosizioniTargetRIBINV'])
        return lista_fondi

    def op_servizi_get_elenco_ribinv(self, cod_cliente, dettaglio_from):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoRIBINV

        SEND
        <codCliente>int</codCliente>
        <DettaglioFrom>string</DettaglioFrom>

        RETURN
        <OpServizi_GetElencoRIBINVNOTEXResult>
            <PosizioniTargetRIBINV>
                <RAPPORTO>long</RAPPORTO>
                <NUMCONTRATTO>string</NUMCONTRATTO>
                <NUMCONTRATTODESCR>string</NUMCONTRATTODESCR>
                <CODFONDO>string</CODFONDO>
                <DESCRFONDO>string</DESCRFONDO>
                <CODTC>string</CODTC>
                <DESCRTC>string</DESCRTC>
                <TIPOPIANO>string</TIPOPIANO>
                <NUMRATEPAC>string</NUMRATEPAC>
                <IMPORTORATA>decimal</IMPORTORATA>
                <PESO>decimal</PESO>
                <CTV>decimal</CTV>
            </PosizioniTargetRIBINV>
        </OpServizi_GetElencoRIBINVEXResult>
        """
        fondi = self.client.OpServizi_GetElencoRIBINV(
            cod_cliente, dettaglio_from
        )['OpServizi_GetElencoRIBINVResult']
        lista_fondi = []
        for fondo in fondi:
            lista_fondi.append(fondo['PosizioniTargetRIBINV'])
        return lista_fondi

    def op_servizi_get_elenco_tcpic(self, codice_cliente, codice_rapporto):
        """
        http://93.57.3.196:8282/service.asmx?op=OpServizi_GetElencoTCPIC

        SEND
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>

        RETURN
        <OpServizi_TCPIC>
            <NumContratto>long</NumContratto>
            <NumContrattoDescr>long</NumContrattoDescr>
        </OpServizi_TCPIC>
        """
        return self.client.OpServizi_GetElencoTCPIC(codice_cliente, codice_rapporto)['OpServizi_GetElencoTCPICResult']
