# -*- coding: utf-8 -*-
"""
views module for acomea.app application
"""
import datetime
import locale
from decimal import Decimal

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http import (
    HttpResponseServerError, HttpResponseRedirect,
    HttpResponseNotFound, HttpResponseForbidden
)
from django.db.models import Sum
from django.core.exceptions import ObjectDoesNotExist
from django.utils.decorators import method_decorator
from django.views.generic import DetailView

from jmb.core.views.generic.create_oo_file import create_file

from acomea.app.models import Servizio, Applicazione
from acomea.app.soap import OpServizi

###################################
## VIEWS                         ##
###################################

@login_required
def eliminazione_servizio_inserito(request, numero_servizio, codice_fiscale):
    """
    delete inserted service
    """
    redirect_url = request.META.get('HTTP_REFERER', '/')
    try:
        servizio = OpServizi()
        result = servizio.op_servizi_annulla_servizio_web(numero_servizio, codice_fiscale)
        if not result:
            messages.error(
                request,
                "Non è possibile effettuare l'operazione richiesta."
            )
    except Exception, e:
        messages.error(
            request,
            "Non è possibile effettuare l'operazione richiesta: %s" % e
        )
    return HttpResponseRedirect(redirect_url)

@login_required
def revoca_servizio_attivo(request, numero_servizio, tipo_servizio):
    """
    delete inserted service
    """
    redirect_url = request.META.get('HTTP_REFERER', '/')
    try:
        servizio = OpServizi()
        simulazione = "S"
        if request.user.profilo.rac.is_cliente:
            codice_fiscale = request.user.profilo.rac.codice_fiscale
            simulazione = "N"

        elif request.user.profilo.rac.is_acomea:
            codice_fiscale = request.session.get('codice_fiscale', '')
        else:
            messages.error(
                request,
                "Non è possibile effettuare l'operazione richiesta."
            )
            return HttpResponseRedirect(redirect_url)

        result = servizio.op_servizi_revoca_servizio_attivo(
            numero_servizio, codice_fiscale,
            simulazione, tipo_servizio
        )
        if not result:
            messages.error(
                request,
                "Non è possibile effettuare l'operazione richiesta."
            )
            return HttpResponseRedirect(redirect_url)

    except Exception, e:
        messages.error(
            request,
            "Non è possibile effettuare l'operazione richiesta: %s" % e
        )
        return HttpResponseRedirect(redirect_url)

    messages.success(
        request,
        "revoca del servizio %s inoltrata correttamente" % numero_servizio
    )
    return HttpResponseRedirect(redirect_url)


class PDFResponseMixin(object):
    """
    PDF response mixin class. Add extra features to ServizioPrintAppView
    """
    def render_to_response(self, context, **response_kwargs):
        """
        return an HttpResponse with pdf as attachment, or a HttpResponseNotFound
        """
        self.template_name, self.servizio_type = self._get_odt_template_and_servizio_type(context)
        if not self.template_name:
            return HttpResponseNotFound(
                "<h4>%s .odt template not found</h4>" % self.servizio_type
            )
        return create_file(
            self.request,
            file_model=self.template_name,
            file_name="riepilogo %s" % self.servizio_type,
            file_type='pdf',
            extra_context=self._get_response_dict(context)
        )

    def _get_odt_template_and_servizio_type(self, context):
        """
        returns a tuple containing the ODT template name and servizio code (e.g. 'SWIPRO')
        """
        try:
            servizio_type = context.get('object').tipo_operazione
            template_name = settings.APP_SERVIZIO_STAMPA_APP_TEMPLATES_DICT.get(servizio_type, None)
            return template_name, servizio_type
        except (IndexError, AttributeError):
            return HttpResponseServerError()

    def _get_response_dict(self, context):
        """
        return response dictionary containing object (Servizio instance) and extra parameters
        """
        response_dict = {}
        obj = context.get('object')
        response_dict.update({'object': obj})
        fine_servizio_mese = getattr(obj, 'fine_servizio_mese', None)
        if fine_servizio_mese and fine_servizio_mese != 0:
            try:
                now = datetime.datetime.now()
                # setting locale to IT to print
                # localized date string
                locale.setlocale(locale.LC_ALL, 'it_IT.utf8')
                end_service_date = datetime.datetime(
                    now.year,
                    int(fine_servizio_mese),
                    now.day
                )
                response_dict.update({
                    'fine_servizio_mese_formatted': end_service_date.strftime("%B")
                })
            except TypeError, err:
                pass
            except locale.Error, lerr:
                pass

        # patrimonio from sum: used in RIBINV, CARDIE
        patrimonio_from_sum = obj.posizioni_from.aggregate(
            Sum('patrimonio_from')
        ).get('patrimonio_from__sum') or 0
        response_dict.update({
            'patrimonio_from_sum': patrimonio_from_sum
        })
        # allocazione calcolata A/B: used in CARDIE
        if patrimonio_from_sum > 0:
            estimated_allocazione_group_a = ((patrimonio_from_sum/100)*obj.car_die_allocazione_a)
            estimated_allocazione_group_b = ((patrimonio_from_sum/100)*obj.car_die_allocazione_b)
            response_dict.update({
                'estimated_allocazione_group_a': estimated_allocazione_group_a.quantize(Decimal('.01')),
                'estimated_allocazione_group_b': estimated_allocazione_group_b.quantize(Decimal('.01'))
            })

        return response_dict


class ServizioPrintAppView(PDFResponseMixin, DetailView):
    """
    Detail view for app printing
    """
    model = Servizio

    def get(self, request, *args, **kwargs):
        """
        get() override with some extra-checks (prevent unallowed prints)
        """
        try:
            logged_in_user = request.user
            if not logged_in_user.profilo.id == self.get_object().profilo.id:
                return HttpResponseForbidden("<h4>Not authorized for this task</h4>")
        except (ObjectDoesNotExist, AttributeError) as e:
            return HttpResponseServerError(
                "error: %s" % e.message
            )
        return super(ServizioPrintAppView, self).get(request, *args, **kwargs)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """
        override for CBV login_required decorator feature implementation
        """
        return super(ServizioPrintAppView, self).dispatch(request, *args, **kwargs)
